
/*
 * Author: Radovan Cerveny
 */

#ifndef _STRINGOLOGY_BACKWARD_NONDETERMINISTIC_DAWG_MATCHING_H_
#define _STRINGOLOGY_BACKWARD_NONDETERMINISTIC_DAWG_MATCHING_H_

#include <string/String.h>
#include <string/StringFeatures.h>
#include <core/multipleDispatch.hpp>

#include <set>

namespace stringology {

namespace exact {

/**
 * Implementation of Backward Nondeterministic DAWG Matching using bit parallelism with 64bit bitmask and brute force switch for longer patterns.
 */
class BackwardNondeterministicDAWGMatching : public std::DoubleDispatch < std::set < unsigned >, string::StringBase, string::StringBase > {
public:
    /**
     * Search for pattern in linear string.
     * @return set set of occurences
     */
    static std::set < unsigned > match ( const string::String & subject, const string::String & pattern );

    static std::set < unsigned > match ( const string::LinearString & subject, const string::LinearString & pattern );

    static BackwardNondeterministicDAWGMatching & getInstance ( ) {
        static BackwardNondeterministicDAWGMatching res;

        return res;
    }

};

} /* namespace exact */

} /* namespace stringology */

#endif /* _STRINGOLOGY_BACKWARD_NONDETERMINISTIC_DAWG_MATCHING_H_ */
